#pragma once
#include "token.hpp"

namespace jlog {
    class Timestamp {
    public:
        Timestamp();
    public:
        std::chrono::year Year() {return y;};
        std::chrono::month Month() {return m;};
        std::chrono::day Day() {return d;}
        std::chrono::duration<long, std::ratio<3600>> Hour() {return h;};
        std::chrono::duration<long, std::ratio<60>> Minute() {return M;};
        std::chrono::duration<long> Second() {return s;};
        std::chrono::duration<long, std::ratio<1, 1000>> Millisecond() {return ms;};
    private:
        std::chrono::year y;
        std::chrono::month m;
        std::chrono::day d;
        std::chrono::duration<long, std::ratio<3600>> h;
        std::chrono::duration<long, std::ratio<60>> M;
        std::chrono::duration<long> s;
        std::chrono::duration<long, std::ratio<1, 1000>> ms;
    };

    std::vector<token> timestamp_format(Timestamp tstamp);

    /// Returns a string timestamp in the format of 'YYYY-MM-DD hh:mm:ss.ms'
    std::string get_timestamp();

    /// Returns a pseudo-"stacktrace" formatted sequence of tokens.
    /// @param func The function name/signature to trace back to. Should be provided by a __FUNCTION__ macro variant.
    /// @param file The file name to trace back to. Should be provided by a __FILE__ macro variant.
    /// @param line The source-code line to trace back to. Should be provided by a __LINE__ macro variant.
        std::vector<token> trace_format(
                const std::string &func,
                const std::string &file,
                int line);

    /// Returns a formatted sequence of tokens given a severity and message.
    /// @param severity_name The severity tag to prefix to the message. Could theoretically also be a context.
    /// @param message The actual message to include
    /// @param severity_cc The colorcode to assign to the severity.
        std::vector<token> log_format(
                const std::string &severity_name,
                const std::string &message,
                const Color4 &severity_cc = Colors::White);

    /// Returns a more detailed formatted sequence of tokens.
    /// @param severity_name The severity tag to prefix to the message. Could theoretically also be a context.
    /// @param message The actual message to include
    /// @param func The function name/signature to trace back to. Should be provided by a __FUNCTION__ macro variant.
    /// @param file The file name to trace back to. Should be provided by a __FILE__ macro variant.
    /// @param line The source-code line to trace back to. Should be provided by a __LINE__ macro variant.
    /// @param severity_cc The colorcode to assign to the severity.
        std::vector<token> log_format(
                const std::string &severity_name,
                const std::string &message,
                const std::string &func,
                const std::string &file,
                int line,
                const Color4 &severity_cc = Colors::White);

    // Generic token for line ending
    //const token endl = {.content = std::endl, .delimiter = ""};
}