#pragma once

#include <functional>
#include <mcolor.h>
#include "Colors.hpp"

namespace jlog {

    using namespace mcolor;

    /// A single piece of a logging message, with color code, content, and delimiter
    /// These are strung together to build full logger messages in a flexible manner.
    struct token {
        Color4 colorCode = Colors::White;
        std::string content = "";
        std::string delimiter = "[]";
    };

    //class TokenStringer {
    //public:
    //    virtual std::string Stringer(std::vector<token> ts) = 0;
    //};

    // I did these after reading the Bjarne book.
    // May or may not be a good idea, but it looks cool(?)
    class WithColor {
    public:
        static std::string Stringer(std::vector<token> ts) {
            std::string msg;
            for (const token &t: ts) {
                if (!t.delimiter.empty()) {
                    msg += std::format("{}{}{}{}{} ",
                                       mcolor::toEscapeCode(t.colorCode),
                                       t.delimiter[0], t.content,
                                       t.delimiter[1],
                                       mcolor::toEscapeCode(AnsiColor::RESET));
                } else {
                    msg += std::format("{}{}{} ",
                                       mcolor::toEscapeCode(t.colorCode),
                                       t.content,
                                       mcolor::toEscapeCode(AnsiColor::RESET));
                }
            }

            return msg;
        };
    };

    class WithoutColor {
    public:
        static std::string Stringer(std::vector<token> ts) {
            std::string msg;
            for (const token &t: ts) {
                if (!t.delimiter.empty())
                    msg += std::format("{}{}{} ", t.delimiter[0], t.content, t.delimiter[1]);
                else
                    msg += t.content + " ";
            }
            return msg;
        }
    };

    // By default we use the WithColor token stringer
    template<class S = WithColor>
    std::string TokensToString(std::vector<token> ts) {
        return S::Stringer(ts);
    }

    /// These are helper functions designed to be wrapped around for easier custom logger building.
    std::string toks2msg(std::vector<token> tokens, std::function<std::string(token)>);
    std::string consoleMsgFormatter(token t);
    std::string logfileMsgFormatter(token t);
    std::string toks2consoleMsg(std::vector<token> tokens);
    std::string toks2logfileMsg(std::vector<token> tokens);
}
