#include <string>
#include <format>
#include <vector>
#include <chrono>
#include "mcolor.h"
#include "jlog/formatter.hpp"
#include "jlog/token.hpp"

namespace jlog {

    Timestamp::Timestamp() {
        auto const timestamp = std::chrono::current_zone()->to_local(std::chrono::system_clock::now());
        auto dp = floor<std::chrono::days>(timestamp);
        std::chrono::year_month_day ymd{floor<std::chrono::days>(timestamp)};
        std::chrono::hh_mm_ss time{floor<std::chrono::milliseconds>(timestamp - dp)};
        y = ymd.year();
        m = ymd.month();
        d = ymd.day();
        h = time.hours();
        M = time.minutes();
        s = time.seconds();
        ms = time.subseconds();
    }

    // [2024-Aug-14 12:0:58.815]
    std::string get_timestamp() {
        using namespace std::chrono;
        auto const timestamp = current_zone()->to_local(system_clock::now());
        auto dp = floor<days>(timestamp);
        year_month_day ymd{floor<days>(timestamp)};
        hh_mm_ss time{floor<milliseconds>(timestamp - dp)};
        auto y = ymd.year();
        auto m = ymd.month();
        auto d = ymd.day();
        auto h = time.hours();
        auto M = time.minutes();
        auto s = time.seconds();
        auto ms = time.subseconds();
        return std::format("{}-{}-{} {}:{}:{}.{}", y, m, d, h.count(), M.count(), s.count(), ms.count());
    }
    //[2024-Aug-14 12:0:58.815]
    std::vector<token> timestamp_format(Timestamp tstamp) {
        return std::vector<token>{{.content = std::format("{}-{}-{} {}:{}:{}.{}", tstamp.Year(), tstamp.Month(), tstamp.Day(), tstamp.Hour().count(), tstamp.Minute().count(), tstamp.Second().count(), tstamp.Millisecond().count())}};
    }

    std::vector<token> trace_format(
            const std::string& func,
            const std::string& file,
            int line)
    {
        auto trace = token{.content = func};
        auto filedata = token{.content = std::format("{}:{}", file, line)};
        return {trace, filedata};
    }

    std::vector<token> log_format(
            const std::string& severity_name,
            const std::string& message,
            const Color4& severity_cc)
    {
        auto severity = token{.colorCode = severity_cc, .content = severity_name};
        auto content = token{.content = message, .delimiter = ""};
        return {severity, content};
    }

    std::vector<token> log_format(
            const std::string& severity_name,
            const std::string& message,
            const std::string& func,
            const std::string& file,
            int line,
            const Color4& severity_cc)
    {
        std::vector<token> tokens;
        //auto timestamp = token{.content = jlog::get_timestamp()};
        auto tsf_tokens = jlog::timestamp_format(Timestamp());
        auto tf_tokens = jlog::trace_format(func, file, line);
        auto lf_tokens = jlog::log_format(severity_name, message, severity_cc);
        //tokens.push_back(timestamp);
        tokens.insert(tokens.end(), tsf_tokens.begin(), tsf_tokens.end());
        tokens.insert(tokens.end(), tf_tokens.begin(), tf_tokens.end());
        tokens.insert(tokens.end(), lf_tokens.begin(), lf_tokens.end());
        return tokens;
    }


}