#include <jlog/logger.hpp>
#include <jlog/jlog.hpp>
#include <jlog/formatter.hpp>
#include <jlog/token.hpp>
#include "jlog/io.hpp"

namespace jlog
{
    Logger::Logger(const std::string& context, const Color4& color) {
        this->context = context;
        this->colorcode = color;
    }

    void Logger::Log(const std::string &message, const std::source_location& location) {
        if (!enabled)
            return;

        std::vector<jlog::token> fmt;

        if (trace)
            fmt = log_format(this->context, message, location.function_name(), location.file_name(), location.line(),this->colorcode);
        else
            fmt = log_format(this->context, message, this->colorcode);

        OnLogEvent(fmt);

        //jlog::log_to_console(jlog::toks2consoleMsg(fmt) + '\n');
        //jlog::log_to_file(this->logfile, jlog::toks2logfileMsg(fmt) + '\n');
        //jlog::log_to_console(jlog::TokensToString(fmt) + '\n');
        //jlog::log_to_file(logfile, jlog::TokensToString<WithoutColor>(fmt) + '\n');
        LogToConsole(fmt);
        LogToConsole("\n");
        //LogToConsole(std::vector<token>{jlog::endl});
        LogToFile(logfile, fmt);
        LogToFile(logfile, "\n");
    }

    void Logger::SetEnabled(bool state) {
        this->enabled = state;
    }
    bool Logger::Enabled() { return this->enabled; }

    void Logger::LogFile(const std::string& f) {
        this->logfile = f;
    }
    std::string Logger::LogFile() { return this->logfile; }

    //void Logger::SetColorCode(AnsiColor cc) { this->colorcode = cc; }
    //AnsiColor Logger::GetColorCode() { return this->colorcode; }
    void Logger::SetColorCode(Color4 cc) { this->colorcode = cc; }
    Color4 Logger::GetColorCode() { return this->colorcode; }

    std::string Logger::Context() { return this->context; }

    void Logger::operator()(const std::string &message, const std::source_location& location) {
        Log(message, location);
    }

    void Logger::SetTraceback(bool enabled) {
        trace = enabled;
    }

    void Logger::Enable() { SetEnabled(true);}

    void Logger::Disable() { SetEnabled(false); }
}